class TalltyImportExport::ExportPayload
  attr_reader :value, :context

  def initialize(val, headers:, &value_handler)
    @value = TalltyImportExport::ExportPayload::Value.new(val)
    @headers = headers.items # TalltyImportExport::Attr::ExportHeaderItem array
    @flatten_headers = headers.flatten_value
    @value_handler = value_handler
    @context = {
      # header_seq_to_cell_seq_to_header_seq_to_value_mapping: {},
      value_seq_to_header_seq_to_value: {},
      # header_seq_to_header_mapping: @headers.reduce({}) do |out, header|
      #   out[header.seq] = header
      #   out
      # end,
      # header_seq_to_children_mapping: @headers.reduce({}) do |out, header|
      #   out[header.seq] = header.children
      #   out
      # end,
      flatten_headers: @flatten_headers,
    }
  end

  def lines
    cell_columns = @headers.map do |header|
      TalltyImportExport::ExportPayload::CellColumn.new(
        header,
        self.value,
        @context,
        &@value_handler
      )
    end

    max_height_cell_column = cell_columns.map(&:flatten_cells).max do |a, b|
      a.count <=> b.count
    end

    max_height_cell_column.map(&:grow_to_line)
  end
end

class TalltyImportExport::ExportPayload::Value
  attr_reader :value, :seq_chain

  def initialize val, seq_chain=[]
    @value = val
    @seq_chain = [*seq_chain, seq]
    # flatten_cells.each do |cell|
    #   context[:cell_seq_to_header_seq_to_value][cell.seq] = {} unless context[:cell_seq_to_header_seq_to_value][cell.seq]
    #   context[:cell_seq_to_header_seq_to_value][cell.seq][header.seq] = value
  end

  def seq
    @seq ||= SecureRandom.uuid
  end
end

# 一个 竖列
class TalltyImportExport::ExportPayload::CellColumn
  attr_reader :header, :payload, :context, :cell_clusters, :flatten_cells

  def initialize header, value, context, &value_handler
    @header = header
    @value = value
    @context = context
    @cell_clusters = []
    @value_handler = value_handler

    if header.children
      @cell_clusters = @value.value[header.key.to_sym].map do |val|
        TalltyImportExport::ExportPayload::Cell.new(
          @header,
          TalltyImportExport::ExportPayload::Value.new(val, @value.seq_chain),
          @context,
          &@value_handler
        )
      end
    else
      @cell_clusters = [
        TalltyImportExport::ExportPayload::Cell.new(
          @header,
          TalltyImportExport::ExportPayload::Value.new(
            @value_handler.call(@value.value, @header.as_json.symbolize_keys),
            @value.seq_chain,
          ),
          @context,
          &@value_handler
        )
      ]
    end

    @flatten_cells = []

    cell_divise

  end

  def cell_divise
    @flatten_cells = @cell_clusters.map do |cell_cluster|
      cell_cluster.divise.cell_cluster
    end.flatten
    self
  end
end

class TalltyImportExport::ExportPayload::Cell
  attr_accessor :cell_cluster, :val, :header, :parent_path

  # cell_cluster 是表格纵向 集合
  # cell_the_same_line 是表格横向 集合
  def initialize header, value, context, parent_path=[], &value_handler
    @value = value
    @header = header
    @value_handler = value_handler

    # context
    @context = context

    value.seq_chain.each do |s|
      unless (@context[:value_seq_to_header_seq_to_value][s])
        @context[:value_seq_to_header_seq_to_value][s] = {}
      end
      @context[:value_seq_to_header_seq_to_value][s][header.seq] = value # NOTE: value 对象
    end

    # parent
    @parent_path = parent_path

    @cell_cluster = []
  end

  def seq
    @seq ||= SecureRandom.uuid
  end

  def heredity_chain
    [*self.parent_path, self]
  end

  def divise
    if @header.children.present?
      new_cells = @header.children.map do |child_header|
        if (child_header.children.present?)
          val = @value.value[child_header.key.to_sym]
          val.map do |val|
            TalltyImportExport::ExportPayload::Cell.new(
              child_header,
              TalltyImportExport::ExportPayload::Value.new(val, @value.seq_chain),
              @context,
              [*self.parent_path, self],
              &@value_handler
            )
          end
        else
          val = @value_handler.call(@value.value, child_header.as_json.symbolize_keys)
          TalltyImportExport::ExportPayload::Cell.new(
            child_header,
            TalltyImportExport::ExportPayload::Value.new(val, @value.seq_chain),
            @context,
            [*self.parent_path, self],
            &@value_handler
          )
        end
      end.flatten
      new_cells.each(&:divise)
      @cell_cluster.concat(new_cells.map(&:cell_cluster))
    else
      @cell_cluster = [self]
    end

    self
  end

  def grow_to_line
    @context[:flatten_headers].map do |header|
      target_val = nil

      @value.seq_chain.each do |seq|
        if @context[:value_seq_to_header_seq_to_value][seq]&.[](header.seq)
          target_val = @context[:value_seq_to_header_seq_to_value][seq][header.seq]
        end
      end

      target_val
    end
  end
end
